using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class AppSettings
{

    public string? BuildNumber { get; set; }
    public string? Company { get; set; }
    public string? Destination { get; set; }
    public string? GitCommitSeven { get; set; }
    public int? MillisecondsDelay { get; set; }
    public string? WatchDirectory { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderAppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    private static Models.AppSettings Get(AppSettings? appSettings)
    {
        Models.AppSettings result;
        if (appSettings is null) throw new NullReferenceException(nameof(appSettings));
        if (appSettings.BuildNumber is null) throw new NullReferenceException(nameof(BuildNumber));
        if (appSettings.Company is null) throw new NullReferenceException(nameof(Company));
        if (appSettings.Destination is null) throw new NullReferenceException(nameof(Destination));
        if (appSettings.GitCommitSeven is null) throw new NullReferenceException(nameof(GitCommitSeven));
        if (appSettings.MillisecondsDelay is null) throw new NullReferenceException(nameof(MillisecondsDelay));
        if (appSettings.WatchDirectory is null) throw new NullReferenceException(nameof(WatchDirectory));
        result = new(appSettings.BuildNumber,
                     appSettings.Company,
                     appSettings.Destination,
                     appSettings.GitCommitSeven,
                     appSettings.MillisecondsDelay.Value,
                     appSettings.WatchDirectory);
        return result;
    }

    public static Models.AppSettings Get(IConfiguration configuration)
    {
        Models.AppSettings result;
#pragma warning disable IL3050, IL2026
        AppSettings? appSettings = configuration.Get<AppSettings>();
#pragma warning restore IL3050, IL2026
        result = Get(appSettings);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class BinderAppSettingsSourceGenerationContext : JsonSerializerContext
{
}